@@ -68,8 +68,13 @@
 #undef RPA_ENABLE
 
 /* Define if you want OPIE support compiled in */
-#undef OPIE_ENABLE
+#undef OPIE
 
+/* Define if you want IPv6 support compiled in */
+#undef INET6
+
+/* Define if you want network security support compiled in */
+#undef NET_SECURITY
 
 /* Leave that blank line there!!  Autoheader needs it.
    If you're adding to this file, keep in mind:
