@@ -33,10 +33,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #endif
 #if defined(HAVE_ALLOCA_H)
 #include <alloca.h>
+#else
+#ifdef _AIX
+ #pragma alloca
+#endif
 #endif
 
-#if HAVE_VPRINTF || HAVE_DOPRNT || _LIBC
-# if __STDC__
+#if HAVE_VPRINTF || HAVE_DOPRNT || _LIBC || HAVE_STDARG_H
+# if HAVE_STDARG_H
 #  include <stdarg.h>
 #  define VA_START(args, lastarg) va_start(args, lastarg)
 # else
@@ -55,12 +59,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 void exit ();
 #endif
 
-#include "fetchmail.h"
-
 #ifndef _
 # define _(String) String
 #endif
 
+#include "fetchmail.h"
+#define MALLOC(n)	xmalloc(n)	
+#define REALLOC(n,s)	xrealloc(n,s)	
+
 /* If NULL, error will flush stdout, then print on stderr the program
    name, a colon and a space.  Otherwise, error will call this
    function without parameters instead.  */
@@ -75,10 +81,12 @@ static unsigned int partial_message_size = 0;
 static unsigned int partial_message_size_used = 0;
 static char *partial_message;
 static unsigned use_stderr;
+static unsigned int use_syslog;
 
 /* This variable is incremented each time `error' is called.  */
 unsigned int error_message_count;
 
+
 #ifdef _LIBC
 /* In the GNU C library, there is a predefined variable for this.  */
 
@@ -118,13 +126,13 @@ private_strerror (errnum)
 /* VARARGS */
 
 void
-#if defined(VA_START) && __STDC__
+#ifdef HAVE_STDARG_H
 error (int status, int errnum, const char *message, ...)
 #else
 error (status, errnum, message, va_alist)
      int status;
      int errnum;
-     char *message;
+     const char *message;
      va_dcl
 #endif
 {
@@ -224,7 +232,7 @@ error (status, errnum, message, va_alist)
       fprintf (stderr, message, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif
 
-      if (errnum) {
+      if (errnum && errnum != -1) {
 	char *tmps = strerror(errnum);
 	if (tmps) {
 	  fprintf (stderr, ": %s", tmps);
@@ -242,14 +250,32 @@ error (status, errnum, message, va_alist)
 }
 
 /*
- * Calling error_init(TRUE) causes error_build and error_complete to write
+ * Calling error_init(1) causes error_build and error_complete to write
  * to stderr without buffering.  This is needed for the ticker dots to
  * work correctly.
  */
-void error_init(foreground)
-int foreground;
+void error_init(int mode)
 {
-    use_stderr = foreground;
+    switch(mode)
+    {
+    case 0:			/* stderr, buffered */
+    default:
+	use_stderr = 0;
+	use_syslog = 0;
+	break;
+
+    case 1:			/* stderr, unbuffered */
+	use_stderr = 1;
+	use_syslog = 0;
+	break;
+
+#ifdef HAVE_SYSLOG
+    case -1:			/* syslogd */
+	use_stderr = 0;
+	use_syslog = 1;
+	break;
+#endif /* HAVE_SYSLOG */
+    }
 }
 
 /* Build an error message by appending MESSAGE, which is a printf-style
@@ -263,11 +289,11 @@ int foreground;
 /* VARARGS */
 
 void
-#if defined(VA_START) && __STDC__
+#ifdef HAVE_STDARG_H
 error_build (const char *message, ...)
 #else
 error_build (message, va_alist)
-     char *message;
+     const char *message;
      va_dcl
 #endif
 {
@@ -281,13 +307,13 @@ error_build (message, va_alist)
     {
       partial_message_size_used = 0;
       partial_message_size = 2048;
-      partial_message = xmalloc (partial_message_size);
+      partial_message = MALLOC (partial_message_size);
     }
   else
     if (partial_message_size - partial_message_size_used < 1024)
       {
         partial_message_size += 2048;
-        partial_message = xrealloc (partial_message, partial_message_size);
+        partial_message = REALLOC (partial_message, partial_message_size);
       }
 
 #if defined(VA_START)
@@ -306,7 +332,7 @@ error_build (message, va_alist)
 	}
 
       partial_message_size += 2048;
-      partial_message = xrealloc (partial_message, partial_message_size);
+      partial_message = REALLOC (partial_message, partial_message_size);
     }
 #else
   vsprintf (partial_message + partial_message_size_used, message, args);
@@ -335,7 +361,7 @@ error_build (message, va_alist)
 	}
 
       partial_message_size += 2048;
-      partial_message = xrealloc (partial_message, partial_message_size);
+      partial_message = REALLOC (partial_message, partial_message_size);
     }
 #else
   sprintf (partial_message + partial_message_size_used, message, a1, a2, a3, a4, a5, a6, a7, a8);
@@ -363,13 +389,13 @@ error_build (message, va_alist)
 /* VARARGS */
 
 void
-#if defined(VA_START) && __STDC__
+#ifdef HAVE_STDARG_H
 error_complete (int status, int errnum, const char *message, ...)
 #else
 error_complete (status, errnum, message, va_alist)
      int status;
      int errnum;
-     char *message;
+     const char *message;
      va_dcl
 #endif
 {
@@ -383,13 +409,13 @@ error_complete (status, errnum, message, va_alist)
     {
       partial_message_size_used = 0;
       partial_message_size = 2048;
-      partial_message = xmalloc (partial_message_size);
+      partial_message = MALLOC (partial_message_size);
     }
   else
     if (partial_message_size - partial_message_size_used < 1024)
       {
         partial_message_size += 2048;
-        partial_message = xrealloc (partial_message, partial_message_size);
+        partial_message = REALLOC (partial_message, partial_message_size);
       }
 
 #if defined(VA_START)
@@ -408,7 +434,7 @@ error_complete (status, errnum, message, va_alist)
 	}
 
       partial_message_size += 2048;
-      partial_message = xrealloc (partial_message, partial_message_size);
+      partial_message = REALLOC (partial_message, partial_message_size);
     }
 #else
   vsprintf (partial_message + partial_message_size_used, message, args);
@@ -437,7 +463,7 @@ error_complete (status, errnum, message, va_alist)
 	}
 
       partial_message_size += 2048;
-      partial_message = xrealloc (partial_message, partial_message_size);
+      partial_message = REALLOC (partial_message, partial_message_size);
     }
 #else
   sprintf (partial_message + partial_message_size_used, message, a1, a2, a3, a4, a5, a6, a7, a8);
@@ -478,7 +504,7 @@ error_complete (status, errnum, message, va_alist)
 int error_one_per_line;
 
 void
-#if defined(VA_START) && __STDC__
+#ifdef HAVE_STDARG_H
 error_at_line (int status, int errnum, const char *file_name,
 	       unsigned int line_number, const char *message, ...)
 #else
@@ -487,7 +513,7 @@ error_at_line (status, errnum, file_name, line_number, message, va_alist)
      int errnum;
      const char *file_name;
      unsigned int line_number;
-     char *message;
+     const char *message;
      va_dcl
 #endif
 {
