@@ -31,7 +31,9 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 
 #ifdef HAVE_GETHOSTBYNAME
 #include <netdb.h>
@@ -43,8 +45,6 @@
 #include "getopt.h"
 #include "netrc.h"
 
-#define DROPDEAD	6	/* maximum bad socket opens */
-
 #ifndef ENETUNREACH
 #define ENETUNREACH   128       /* Interactive doesn't know this */
 #endif /* ENETUNREACH */
@@ -72,6 +72,11 @@ char *home;
 char *fetchmailhost;	/* the name of the host running fetchmail */
 char *program_name;	/* the name to prefix error messages with */
 
+#if NET_SECURITY
+void *request = NULL;
+int requestlen = 0;
+#endif /* NET_SECURITY */
+
 static char *lockfile;		/* name of lockfile */
 static int querystatus;		/* status of query */
 static int successes;		/* count number of successful polls */
@@ -79,6 +84,7 @@ static int lastsig;		/* last signal received */
 
 static void termhook();		/* forward declaration of exit hook */
 
+
 RETSIGTYPE donothing(sig) int sig; {signal(sig, donothing); lastsig = sig;}
 
 #ifdef HAVE_ON_EXIT
@@ -91,6 +97,19 @@ static void unlockit(void)
     unlink(lockfile);
 }
 
+#ifdef __EMX__
+/* Various EMX-specific definitions */
+int itimerflag;
+void itimerthread(void* dummy) {
+  if (outlevel == O_VERBOSE)
+    fprintf(stderr, "fetchmail: thread sleeping for %d sec.\n", poll_interval);
+  while(1) {
+    _sleep2(poll_interval*1000);
+    kill((getpid()), SIGALRM);
+  }
+}
+#endif
+
 int main (int argc, char **argv)
 {
     int st, bkgd = FALSE;
@@ -115,7 +134,7 @@ int main (int argc, char **argv)
 	exit(PS_SYNTAX);
 
     /* this hint to stdio should help messages come out in the right order */
-    setvbuf(stdout, NULL, _IOLBF, POPBUFSIZE);
+    setvbuf(stdout, NULL, _IOLBF, MSGBUFSIZE);
 
     if (versioninfo)
     {
@@ -135,6 +154,15 @@ int main (int argc, char **argv)
 #ifndef ETRN_ENABLE
 	printf("-ETRN");
 #endif /* ETRN_ENABLE */
+#if OPIE
+	printf("+OPIE");
+#endif /* OPIE */
+#if INET6
+	printf("+INET6");
+#endif /* INET6 */
+#if NET_SECURITY
+	printf("+NETSEC");
+#endif /* NET_SECURITY */
 	putchar('\n');
 
 	/* this is an attempt to help remote debugging */
@@ -142,7 +170,7 @@ int main (int argc, char **argv)
     }
 
     /* avoid parsing the config file if all we're doing is killing a daemon */ 
-    if (!quitmode)
+    if (!(quitmode && argc == 2))
 	implicitmode = load_params(argc, argv, optind);
 
     /* set up to do lock protocol */
@@ -150,7 +178,7 @@ int main (int argc, char **argv)
 	sprintf(tmpbuf, "%s/fetchmail.pid", PID_DIR);
     else {
 	strcpy(tmpbuf, home);
-	strcat(tmpbuf, "/.fetchmail");
+	strcat(tmpbuf, "/.fetchmail.pid");
     }
 
     /* perhaps we just want to check options? */
@@ -205,7 +233,7 @@ int main (int argc, char **argv)
     }
 
     /* if no mail servers listed and nothing in background, we're done */
-    if (!quitmode && pid == -1 && querylist == NULL) {
+    if (!(quitmode && argc == 2) && pid == -1 && querylist == NULL) {
 	(void)fputs("fetchmail: no mailservers have been specified.\n",stderr);
 	exit(PS_SYNTAX);
     }
@@ -216,11 +244,12 @@ int main (int argc, char **argv)
 	if (pid == -1) 
 	{
 	    fprintf(stderr,"fetchmail: no other fetchmail is running\n");
-	    exit(PS_EXCLUDE);
+	    if (argc == 2)
+		exit(PS_EXCLUDE);
 	}
 	else if (kill(pid, SIGTERM) < 0)
 	{
-	    fprintf(stderr,"fetchmail: error killing %s fetchmail at %d.\n",
+	    fprintf(stderr,"fetchmail: error killing %s fetchmail at %d; bailing out.\n",
 		    bkgd ? "background" : "foreground", pid);
 	    exit(PS_EXCLUDE);
 	}
@@ -229,7 +258,10 @@ int main (int argc, char **argv)
 	    fprintf(stderr,"fetchmail: %s fetchmail at %d killed.\n",
 		    bkgd ? "background" : "foreground", pid);
 	    unlink(lockfile);
-	    exit(0);
+	    if (argc == 2)
+		exit(0);
+	    else
+		pid = -1; 
 	}
     }
 
@@ -293,7 +325,10 @@ int main (int argc, char **argv)
     for (ctl = querylist; ctl; ctl = ctl->next)
 	if (ctl->active && !(implicitmode && ctl->server.skip)&&!ctl->password)
 	{
-	    if (ctl->server.preauthenticate == A_KERBEROS_V4 || ctl->server.protocol == P_IMAP_K4)
+	    if (ctl->server.preauthenticate == A_KERBEROS_V4 ||
+		ctl->server.preauthenticate == A_KERBEROS_V5 ||
+		ctl->server.protocol == P_IMAP_K4 ||
+		ctl->server.protocol == P_IMAP_GSS)
 		/* Server won't care what the password is, but there
 		   must be some non-null string here.  */
 		ctl->password = ctl->remotename;
@@ -309,7 +344,7 @@ int main (int argc, char **argv)
 		    ctl->password = xstrdup(p->password);
 	    }
 
-	    if (ctl->server.protocol != P_ETRN && ctl->server.protocol != P_IMAP_K4 && !ctl->password)
+	    if (ctl->server.protocol != P_ETRN && ctl->server.protocol != P_IMAP_K4 && ctl->server.protocol != P_IMAP_GSS && !ctl->password)
 	    {
 		(void) sprintf(tmpbuf, "Enter password for %s@%s: ",
 			       ctl->remotename, ctl->server.pollname);
@@ -322,8 +357,13 @@ int main (int argc, char **argv)
      */
 #if defined(HAVE_SYSLOG)
     if (errors_to_syslog)
+    {
     	openlog(program_name, LOG_PID, LOG_MAIL);
+	error_init(-1);
+    }
+    else
 #endif
+	error_init(poll_interval == 0 && !logfile);
 
     if (poll_interval)
     {
@@ -401,11 +441,11 @@ int main (int argc, char **argv)
 		    }
 		}
 
-#ifdef linux
+#if defined(linux) && !INET6
 		/* interface_approve() does its own error logging */
 		if (!interface_approve(&ctl->server))
 		    continue;
-#endif /* linux */
+#endif /* defined(linux) && !INET6 */
 
 #ifdef HAVE_GETHOSTBYNAME
 		/*
@@ -413,7 +453,9 @@ int main (int argc, char **argv)
 		 * nameserver is still up.  The multidrop case
 		 * (especially) needs it.
 		 */
-		if (ctl->server.preauthenticate==A_KERBEROS_V4 || MULTIDROP(ctl))
+		if (ctl->server.preauthenticate==A_KERBEROS_V4 ||
+		    ctl->server.preauthenticate==A_KERBEROS_V5 ||
+		    MULTIDROP(ctl))
 		{
 		    struct hostent	*namerec;
 
@@ -453,7 +495,7 @@ int main (int argc, char **argv)
 		      update_str_lists(ctl);
 #endif  /* POP3_ENABLE */
 		}
-#ifdef	linux
+#if defined(linux) && !INET6
 		if (ctl->server.monitor)
 		    {
 			/* Allow some time for the link to quiesce.  One
@@ -463,7 +505,7 @@ int main (int argc, char **argv)
 			sleep(3);
 			interface_note_activity(&ctl->server);
 		    }
-#endif
+#endif /* defined(linux) && !INET6 */
 	    }
 	}
 
@@ -518,6 +560,40 @@ int main (int argc, char **argv)
 	     * has to have a BSDoid socket layer to work at all.
 	     */
 	    {
+#ifndef __EMX__
+#ifndef SLEEP_WITH_ALARM	/* not normally on */
+                struct timeval timeout;
+
+                timeout.tv_sec = poll_interval;
+                timeout.tv_usec = 0;
+		lastsig = 0;
+                select(0,0,0,0, &timeout);
+#else
+		/* 
+		 * This code is flaky under Red Hat Linux 5.0, and possibly
+		 * elsewhere.  John Stracke <francis@netscape.com> writes:
+		 *
+		 * The problem seems to be that, after hitting the interval
+		 * timer while talking to the server, the process no longer
+		 * responds to SIGALRM.  I put in printf()s to see when it
+		 * reached the pause() for the poll interval, and I checked
+		 * the return from setitimer(), and everything seemed to be
+		 * working fine, except that the pause() just ignored SIGALRM.
+		 * I thought maybe the itimer wasn't being fired, so I hit
+		 * it with a SIGALRM from the command line, and it ignored
+		 * that, too.  SIGUSR1 woke it up just fine, and it proceeded
+		 * to repoll--but, when the dummy server didn't respond, it
+		 * never timed out, and SIGALRM wouldn't make it.
+		 *
+		 * So the workaround I used is to make it sleep by using
+		 * select() instead of setitimer()/pause().  select() is
+		 * perfectly happy being called with a timeout and
+		 * no file descriptors; it just sleeps until it hits the
+		 * timeout.  The only concern I had was that it might
+		 * implement its timeout with SIGALRM--there are some
+		 * Unices where this is done, because select() is a library
+		 * function--but apparently not.
+		 */
 		struct itimerval ntimeout;
 
 		ntimeout.it_interval.tv_sec = ntimeout.it_interval.tv_usec = 0;
@@ -528,6 +604,13 @@ int main (int argc, char **argv)
 		signal(SIGALRM, donothing);
 		pause();
 		signal(SIGALRM, SIG_IGN);
+#endif
+#else /* EMX */
+		signal(SIGALRM, donothing);
+		_beginthread(itimerthread, NULL, 32768, NULL);
+		pause();
+		signal(SIGALRM, SIG_IGN);
+#endif /* ! EMX */
 		if (lastsig == SIGUSR1
 			|| ((poll_interval && !getuid()) && lastsig == SIGHUP))
 		{
@@ -571,14 +654,13 @@ static int load_params(int argc, char **argv, int optind)
 
     memset(&def_opts, '\0', sizeof(struct query));
     def_opts.smtp_socket = -1;
+    def_opts.smtpaddress = (char *)0;
+    def_opts.antispam = 571;
 
     def_opts.server.protocol = P_AUTO;
     def_opts.server.timeout = CLIENT_TIMEOUT;
     def_opts.remotename = user;
-    save_str(&def_opts.smtphunt, FALSE, fetchmailhost);
-    save_str(&def_opts.smtphunt, FALSE, "localhost");
     def_opts.expunge = 1;
-    def_opts.server.envskip = 0;
 
     /* this builds the host list */
     if (prc_parse_file(rcfile, !versioninfo) != 0)
@@ -630,6 +712,10 @@ static int load_params(int argc, char **argv, int optind)
 	    /* merge in defaults */
 	    optmerge(ctl, &def_opts);
 
+	    /* make sure we have a nonempty host list to forward to */
+	    if (!ctl->smtphunt)
+		save_str(&ctl->smtphunt, FALSE, fetchmailhost);
+
 	    /* keep lusers from shooting themselves in the foot :-) */
 	    if (poll_interval && ctl->limit)
 	    {
@@ -637,29 +723,13 @@ static int load_params(int argc, char **argv, int optind)
 		exit(PS_SYNTAX);
 	    }
 
-	    /* make sure delivery will default to a real local user */
+	    /* if `user' doesn't name a real local user, try to run as root */
 	    if ((pw = getpwnam(user)) == (struct passwd *)NULL)
-	    {
-		fprintf(stderr,
-			"fetchmail: can't set up default delivery to %s\n", user);
-		exit(PS_SYNTAX);	/* has to be from bad rc file */
-	    }
-	    else
-	    {
+		ctl->uid = 0;
+            else
 		ctl->uid = pw->pw_uid;	/* for local delivery via MDA */
-		if (!ctl->localnames)	/* for local delivery via SMTP */
-		    save_str_pair(&ctl->localnames, user, NULL);
-	    }
-
-#if !defined(HAVE_GETHOSTBYNAME) || !defined(HAVE_RES_SEARCH)
-	    /* can't handle multidrop mailboxes unless we can do DNS lookups */
-	    if (ctl->localnames && ctl->localnames->next)
-	    {
-		fputs("fetchmail: can't handle multidrop mailboxes without DNS\n",
-			stderr);
-		exit(PS_SYNTAX);
-	    }
-#endif /* !HAVE_GETHOSTBYNAME || !HAVE_RES_SEARCH */
+	    if (!ctl->localnames)	/* for local delivery via SMTP */
+		save_str_pair(&ctl->localnames, user, NULL);
 
 	    /* this code enables flags to be turned off */
 #define DEFAULT(flag, dflt)	if (flag == FLAG_TRUE)\
@@ -680,6 +750,15 @@ static int load_params(int argc, char **argv, int optind)
 	    DEFAULT(ctl->server.uidl, FALSE);
 #undef DEFAULT
 
+#if !defined(HAVE_GETHOSTBYNAME) || !defined(HAVE_RES_SEARCH)
+	    /* can't handle multidrop mailboxes unless we can do DNS lookups */
+	    if (ctl->localnames && ctl->localnames->next && ctl->server.dns)
+	    {
+		ctl->server.dns = FALSE;
+		fprintf(stderr, "fetchmail: warning: no DNS available to check multidrop fetches from %s\n", ctl->server.pollname);
+	    }
+#endif /* !HAVE_GETHOSTBYNAME || !HAVE_RES_SEARCH */
+
 	    /*
 	     *
 	     * Compute the true name of the mailserver host.  
@@ -714,6 +793,7 @@ static int load_params(int argc, char **argv, int optind)
 	    if (ctl->server.timeout == -1)	
 		ctl->server.timeout = CLIENT_TIMEOUT;
 
+#if !INET6
 	    /* sanity checks */
 	    if (ctl->server.port < 0)
 	    {
@@ -729,6 +809,7 @@ static int load_params(int argc, char **argv, int optind)
 			       ctl->server.pollname);
 		exit(PS_SYNTAX);
 	    }
+#endif /* !INET6 */
 	}
     }
 
@@ -751,7 +832,6 @@ static int load_params(int argc, char **argv, int optind)
     /* check and daemon options are not compatible */
     if (check_only && poll_interval)
 	poll_interval = 0;
-
     return(implicitmode);
 }
 
@@ -866,6 +946,7 @@ static int query_host(struct query *ctl)
 	break;
     case P_IMAP:
     case P_IMAP_K4:
+    case P_IMAP_GSS:
 #ifdef IMAP_ENABLE
 	return(doIMAP(ctl));
 #else
@@ -918,13 +999,25 @@ void dump_params (struct query *ctl)
         else
 	    printf("  Password = '%s'.\n", visbuf(ctl->password));
     if (ctl->server.protocol == P_POP3 
+#if INET6
+		&& !strcmp(ctl->server.service, KPOP_PORT)
+#else /* INET6 */
 		&& ctl->server.port == KPOP_PORT
-		&& ctl->server.preauthenticate == A_KERBEROS_V4)
+#endif /* INET6 */
+		&& (ctl->server.preauthenticate == A_KERBEROS_V4 ||
+		    ctl->server.preauthenticate == A_KERBEROS_V5))
 	printf("  Protocol is KPOP");
     else
 	printf("  Protocol is %s", showproto(ctl->server.protocol));
+#if INET6
+    if (ctl->server.service)
+	printf(" (using service %s)", ctl->server.service);
+    if (ctl->server.netsec)
+	printf(" (using network security options %s)", ctl->server.netsec);
+#else /* INET6 */
     if (ctl->server.port)
 	printf(" (using port %d)", ctl->server.port);
+#endif /* INET6 */
     else if (outlevel == O_VERBOSE)
 	printf(" (using default port)");
     if (ctl->server.uidl)
@@ -933,6 +1026,8 @@ void dump_params (struct query *ctl)
     putchar('\n');
     if (ctl->server.preauthenticate == A_KERBEROS_V4)
 	    printf("  Kerberos V4 preauthentication enabled.\n");
+    if (ctl->server.preauthenticate == A_KERBEROS_V5)
+	    printf("  Kerberos V5 preauthentication enabled.\n");
     if (ctl->server.timeout > 0)
 	printf("  Server nonresponse timeout is %d seconds", ctl->server.timeout);
     if (ctl->server.timeout ==  CLIENT_TIMEOUT)
@@ -975,22 +1070,22 @@ void dump_params (struct query *ctl)
     printf("  Nonempty Status lines will be %s (dropstatus %s)\n",
 	   ctl->dropstatus ? "discarded" : "kept",
 	   ctl->dropstatus ? "on" : "off");
-    if (ctl->limit > 0)
+    if (NUM_NONZERO(ctl->limit))
 	printf("  Message size limit is %d bytes (--limit %d).\n", 
 	       ctl->limit, ctl->limit);
     else if (outlevel == O_VERBOSE)
 	printf("  No message size limit (--limit 0).\n");
-    if (ctl->fetchlimit > 0)
+    if (NUM_NONZERO(ctl->fetchlimit))
 	printf("  Received-message limit is %d (--fetchlimit %d).\n",
 	       ctl->fetchlimit, ctl->fetchlimit);
     else if (outlevel == O_VERBOSE)
 	printf("  No received-message limit (--fetchlimit 0).\n");
-    if (ctl->batchlimit > 0)
+    if (NUM_NONZERO(ctl->batchlimit))
 	printf("  SMTP message batch limit is %d.\n", ctl->batchlimit);
     else if (outlevel == O_VERBOSE)
 	printf("  No SMTP message batch limit (--batchlimit 0).\n");
     if (ctl->server.protocol == P_IMAP)
-	if (ctl->expunge > 0)
+	if (NUM_NONZERO(ctl->expunge))
 	    printf("  Deletion interval between expunges is %d (--expunge %d).\n", ctl->expunge, ctl->expunge);
 	else if (outlevel == O_VERBOSE)
 	    printf("  No expunges (--expunge 0).\n");
@@ -1009,7 +1104,15 @@ void dump_params (struct query *ctl)
 	    	    printf(" (default)");
 	    }
 	printf("\n");
+	if (ctl->smtpaddress)
+	    printf("  Host part of MAIL FROM line will be %s\n",
+		   ctl->smtpaddress);
     }
+    if (ctl->antispam != -1)
+	printf("  Listener SMTP reponse %d will be treated as a spam block\n",
+	       ctl->antispam);
+    else if (outlevel == O_VERBOSE)
+	printf("  Spam-blocking disabled\n");
     if (ctl->preconnect)
 	printf("  Server connection will be brought up with '%s.'\n",
 	       visbuf(ctl->preconnect));
