@@ -26,6 +26,15 @@
 #include <krb.h>
 #endif /* KERBEROS_V4 */
 
+#ifdef GSSAPI
+#include <gssapi/gssapi.h>
+#include <gssapi/gssapi_generic.h>
+#endif
+
+#if OPIE
+#include <opie.h>
+#endif /* OPIE */
+
 #ifndef strstr		/* glibc-2.1 declares this as a macro */
 extern char *strstr();	/* needed on sysV68 R3V7.1. */
 #endif /* strstr */
@@ -36,11 +45,12 @@ extern char *strstr();	/* needed on sysV68 R3V7.1. */
 #define IMAP4rev1	1	/* IMAP4 rev 1, RFC2060 */
 
 static int count, seen, recent, unseen, deletions,expunged, imap_version;
+static char capabilities[MSGBUFSIZE+1];
 
 int imap_ok(int sock, char *argbuf)
 /* parse command response */
 {
-    char buf [POPBUFSIZE+1];
+    char buf [MSGBUFSIZE+1];
 
     seen = 0;
     do {
@@ -50,6 +60,8 @@ int imap_ok(int sock, char *argbuf)
 	    return(ok);
 
 	/* interpret untagged status responses */
+	if (strstr(buf, "* CAPABILITY"))
+	    strncpy(capabilities, buf + 12, sizeof(capabilities));
 	if (strstr(buf, "EXISTS"))
 	    count = atoi(buf+2);
 	if (strstr(buf, "RECENT"))
@@ -102,6 +114,69 @@ int imap_ok(int sock, char *argbuf)
     }
 }
 
+#if OPIE
+static int do_otp(int sock, struct query *ctl)
+{
+  int i, rval;
+  char buffer[128];
+  char challenge[OPIE_CHALLENGE_MAX+1];
+  char response[OPIE_RESPONSE_MAX+1];
+
+  gen_send(sock, "AUTHENTICATE X-OTP");
+
+  if (rval = gen_recv(sock, buffer, sizeof(buffer)))
+    return rval;
+
+  if ((i = from64tobits(challenge, buffer)) < 0) {
+    error(0, -1, "Could not decode initial BASE64 challenge");
+    return PS_AUTHFAIL;
+  };
+
+
+  to64frombits(buffer, ctl->remotename, strlen(ctl->remotename));
+
+  if (outlevel == O_VERBOSE)
+     error(0, 0, "IMAP> %s", buffer);
+  SockWrite(sock, buffer, strlen(buffer));
+  SockWrite(sock, "\r\n", 2);
+
+  if (rval = gen_recv(sock, buffer, sizeof(buffer)))
+    return rval;
+
+  if ((i = from64tobits(challenge, buffer)) < 0) {
+    error(0, -1, "Could not decode OTP challenge");
+    return PS_AUTHFAIL;
+  };
+
+  rval = opiegenerator(challenge, !strcmp(ctl->password, "opie") ? "" : ctl->password, response);
+  if ((rval == -2) && (cmd_daemon == -1)) {
+    char secret[OPIE_SECRET_MAX+1];
+    fprintf(stderr, "Secret pass phrase: ");
+    if (opiereadpass(secret, sizeof(secret), 0))
+      rval = opiegenerator(challenge, secret, response);
+    memset(secret, 0, sizeof(secret));
+  };
+
+  if (rval)
+   return PS_AUTHFAIL;
+
+  to64frombits(buffer, response, strlen(response));
+
+  if (outlevel == O_VERBOSE)
+     error(0, 0, "IMAP> %s", buffer);
+  SockWrite(sock, buffer, strlen(buffer));
+  SockWrite(sock, "\r\n", 2);
+
+  if (rval = gen_recv(sock, buffer, sizeof(buffer)))
+    return rval;
+
+  if (strstr(buffer, "OK"))
+    return PS_SUCCESS;
+  else
+    return PS_AUTHFAIL;
+};
+#endif /* OPIE */
+
 #ifdef KERBEROS_V4
 #if SIZEOF_INT == 4
 typedef	int	int32;
@@ -173,7 +248,7 @@ static int do_rfc1731(int sock, char *truename)
       }
     }
 
-    strncpy(srvrealm, krb_realmofhost(srvinst), (sizeof srvrealm)-1);
+    strncpy(srvrealm, (char *)krb_realmofhost(srvinst), (sizeof srvrealm)-1);
     srvrealm[(sizeof srvrealm)-1] = '\0';
     if (p = strchr(srvinst, '.')) {
       *p = '\0';
@@ -328,44 +403,246 @@ static int do_rfc1731(int sock, char *truename)
 }
 #endif /* KERBEROS_V4 */
 
+#ifdef GSSAPI
+#define GSSAUTH_P_NONE      1
+#define GSSAUTH_P_INTEGRITY 2
+#define GSSAUTH_P_PRIVACY   4
+
+static int do_gssauth(int sock, char *hostname, char *username)
+{
+    gss_buffer_desc request_buf, send_token;
+    gss_buffer_t sec_token;
+    gss_name_t target_name;
+    gss_ctx_id_t context;
+    gss_OID mech_name;
+    gss_qop_t quality;
+    int cflags;
+    OM_uint32 maj_stat, min_stat;
+    char buf1[8192], buf2[8192], server_conf_flags;
+    unsigned long buf_size;
+    int result;
+
+    /* first things first: get an imap ticket for host */
+    sprintf(buf1, "imap@%s", hostname);
+    request_buf.value = buf1;
+    request_buf.length = strlen(buf1) + 1;
+    maj_stat = gss_import_name(&min_stat, &request_buf, gss_nt_service_name,
+        &target_name);
+    if (maj_stat != GSS_S_COMPLETE) {
+        error(0, -1, "Couldn't get service name for [%s]", buf1);
+        return PS_AUTHFAIL;
+    }
+    else if (outlevel == O_VERBOSE) {
+        maj_stat = gss_display_name(&min_stat, target_name, &request_buf,
+            &mech_name);
+        error(0, 0, "Using service name [%s]",request_buf.value);
+        maj_stat = gss_release_buffer(&min_stat, &request_buf);
+    }
+
+    gen_send(sock, "AUTHENTICATE GSSAPI");
+
+    /* upon receipt of the GSSAPI authentication request, server returns
+     * null data ready response. */
+    if (result = gen_recv(sock, buf1, sizeof buf1)) {
+        return result;
+    }
+
+    /* now start the security context initialisation loop... */
+    sec_token = GSS_C_NO_BUFFER;
+    context = GSS_C_NO_CONTEXT;
+    if (outlevel == O_VERBOSE)
+        error(0,0,"Sending credentials");
+    do {
+        maj_stat = gss_init_sec_context(&min_stat, GSS_C_NO_CREDENTIAL, 
+            &context, target_name, NULL, 0, 0, NULL, sec_token, NULL,
+	    &send_token, &cflags, NULL);
+        if (maj_stat!=GSS_S_COMPLETE && maj_stat!=GSS_S_CONTINUE_NEEDED) {
+            error(0, -1,"Error exchanging credentials");
+            gss_release_name(&min_stat, &target_name);
+            /* wake up server and await NO response */
+            SockWrite(sock, "\r\n", 2);
+            if (result = gen_recv(sock, buf1, sizeof buf1))
+                return result;
+            return PS_AUTHFAIL;
+        }
+        to64frombits(buf1, send_token.value, send_token.length);
+        gss_release_buffer(&min_stat, &send_token);
+        SockWrite(sock, buf1, strlen(buf1));
+        SockWrite(sock, "\r\n", 2);
+        if (outlevel == O_VERBOSE)
+            error(0,0,"IMAP> %s", buf1);
+        if (maj_stat == GSS_S_CONTINUE_NEEDED) {
+	    if (result = gen_recv(sock, buf1, sizeof buf1)) {
+	        gss_release_name(&min_stat, &target_name);
+	        return result;
+	    }
+	    request_buf.length = from64tobits(buf2, buf1 + 2);
+	    request_buf.value = buf2;
+	    sec_token = &request_buf;
+        }
+    } while (maj_stat == GSS_S_CONTINUE_NEEDED);
+    gss_release_name(&min_stat, &target_name);
+
+    /* get security flags and buffer size */
+    if (result = gen_recv(sock, buf1, sizeof buf1)) {
+        return result;
+    }
+    request_buf.length = from64tobits(buf2, buf1 + 2);
+    request_buf.value = buf2;
+
+    maj_stat = gss_unwrap(&min_stat, context, &request_buf, &send_token,
+        &cflags, &quality);
+    if (maj_stat != GSS_S_COMPLETE) {
+        error(0,-1,"Couldn't unwrap security level data");
+        gss_release_buffer(&min_stat, &send_token);
+        return PS_AUTHFAIL;
+    }
+    if (outlevel == O_VERBOSE)
+        error(0,0,"Credential exchange complete");
+    /* first octet is security levels supported. We want none, for now */
+    server_conf_flags = ((char *)send_token.value)[0];
+    if ( !(((char *)send_token.value)[0] & GSSAUTH_P_NONE) ) {
+        error(0,-1,"Server requires integrity and/or privacy");
+        gss_release_buffer(&min_stat, &send_token);
+        return PS_AUTHFAIL;
+    }
+    ((char *)send_token.value)[0] = 0;
+    buf_size = ntohl(*((long *)send_token.value));
+    /* we don't care about buffer size if we don't wrap data */
+    gss_release_buffer(&min_stat, &send_token);
+    if (outlevel == O_VERBOSE) {
+        error(0,0,"Unwrapped security level flags: %s%s%s",
+            server_conf_flags & GSSAUTH_P_NONE ? "N" : "-",
+            server_conf_flags & GSSAUTH_P_INTEGRITY ? "I" : "-",
+            server_conf_flags & GSSAUTH_P_PRIVACY ? "C" : "-");
+        error(0,0,"Maximum GSS token size is %ld",buf_size);
+    }
+
+    /* now respond in kind (hack!!!) */
+    buf_size = htonl(buf_size); /* do as they do... only matters if we do enc */
+    memcpy(buf1, &buf_size, 4);
+    buf1[0] = GSSAUTH_P_NONE;
+    strcpy(buf1+4, username); /* server decides if princ is user */
+    request_buf.length = 4 + strlen(username) + 1;
+    request_buf.value = buf1;
+    maj_stat = gss_wrap(&min_stat, context, 0, GSS_C_QOP_DEFAULT, &request_buf,
+        &cflags, &send_token);
+    if (maj_stat != GSS_S_COMPLETE) {
+        error(0,-1,"Error creating security level request");
+        return PS_AUTHFAIL;
+    }
+    to64frombits(buf1, send_token.value, send_token.length);
+    if (outlevel == O_VERBOSE) {
+        error(0,0,"Requesting authorisation as %s", username);
+        error(0,0,"IMAP> %s",buf1);
+    }
+    SockWrite(sock, buf1, strlen(buf1));
+    SockWrite(sock, "\r\n", 2);
+
+    /* we should be done. Get status and finish up */
+    if (result = gen_recv(sock, buf1, sizeof buf1))
+        return result;
+    if (strstr(buf1, "OK")) {
+        /* flush security context */
+        if (outlevel == O_VERBOSE)
+            error(0, 0, "Releasing GSS credentials");
+        maj_stat = gss_delete_sec_context(&min_stat, &context, &send_token);
+        if (maj_stat != GSS_S_COMPLETE) {
+            error(0, -1, "Error releasing credentials");
+            return PS_AUTHFAIL;
+        }
+        /* send_token may contain a notification to the server to flush
+         * credentials. RFC 1731 doesn't specify what to do, and since this
+         * support is only for authentication, we'll assume the server
+         * knows enough to flush its own credentials */
+        return PS_SUCCESS;
+    }
+
+    return PS_AUTHFAIL;
+}	
+#endif /* GSSAPI */
+
+static char *canonify_imap_password(char *passwd)
+/* encode an IMAP password as per RFC1730's quoting conventions */
+{
+    char *result;
+    int i, j;
+
+    result = malloc(2*strlen(passwd));
+    if (!result)
+	return 0;
+
+    j=0;
+    for (i=0; i<strlen(passwd); ++i)
+    {
+	if ((passwd[i] == '\\') || (passwd[i] == '"'))
+	    result[j++] = '\\';
+	result[j++] = passwd[i];
+    }
+    result[j] = '\0';
+
+    return(result);
+}
+
 int imap_getauth(int sock, struct query *ctl, char *greeting)
 /* apply for connection authorization */
 {
-    char capabilities[POPBUFSIZE+1];
     int ok = 0;
 
     /* probe to see if we're running IMAP4 and can use RFC822.PEEK */
-    gen_send(sock, "CAPABILITY");
-    if ((ok = gen_recv(sock, capabilities, sizeof(capabilities))))
-	return(ok);
-    if (strstr(capabilities, "BAD"))
+    capabilities[0] = '\0';
+    if ((ok = gen_transact(sock, "CAPABILITY")) == PS_SUCCESS)
+    {
+	/* UW-IMAP server 10.173 notifies in all caps */
+	if (strstr(capabilities, "IMAP4rev1") || strstr(capabilities, "IMAP4REV1"))
+	{
+	    imap_version = IMAP4rev1;
+	    if (outlevel == O_VERBOSE)
+		error(0, 0, "Protocol identified as IMAP4 rev 1");
+	}
+	else
+	{
+	    imap_version = IMAP4;
+	    if (outlevel == O_VERBOSE)
+	    error(0, 0, "Protocol identified as IMAP4 rev 0");
+	}
+    }
+    else if (ok == PS_ERROR)
     {
 	imap_version = IMAP2;
 	if (outlevel == O_VERBOSE)
 	    error(0, 0, "Protocol identified as IMAP2 or IMAP2BIS");
     }
-    /* UW-IMAP server 10.173 notifies in all caps */
-    else if (strstr(capabilities, "IMAP4rev1") || strstr(capabilities, "IMAP4REV1"))
-    {
-	imap_version = IMAP4rev1;
-	if (outlevel == O_VERBOSE)
-	    error(0, 0, "Protocol identified as IMAP4 rev 1");
-    }
     else
-    {
-	imap_version = IMAP4;
-	if (outlevel == O_VERBOSE)
-	    error(0, 0, "Protocol identified as IMAP4 rev 0");
-    }
+	return(ok);
 
-    /* eat the tail of the CAPABILITY response (if any) */
-    if ((peek_capable = (imap_version >= IMAP4)))
-    {
-	char	scratchbuf[POPBUFSIZE];	/* don't clobber capabilities buffer */
+    peek_capable = (imap_version >= IMAP4);
 
-	if ((ok = gen_recv(sock, scratchbuf, sizeof(scratchbuf))))
-	    return(ok);
+#if OPIE
+    if ((ctl->server.protocol == P_IMAP) && strstr(capabilities, "AUTH=X-OTP")) {
+      if (outlevel == O_VERBOSE)
+        error(0, 0, "OTP authentication is supported");
+      if (do_otp(sock, ctl) == PS_SUCCESS)
+        return PS_SUCCESS;
+    };
+#endif /* OPIE */
+
+#ifdef GSSAPI
+    if (strstr(capabilities, "AUTH=GSSAPI"))
+    {
+        if (ctl->server.protocol == P_IMAP_GSS)
+        {
+            if (outlevel == O_VERBOSE)
+                error(0, 0, "GSS authentication is supported");
+            return do_gssauth(sock, ctl->server.truename, ctl->remotename);
+        }
     }
+    else if (ctl->server.protocol == P_IMAP_GSS)
+    {
+        error(0,-1, "Required GSS capability not supported by server");
+        return(PS_AUTHFAIL);
+    }
+#endif /* GSSAPI */
 
 #ifdef KERBEROS_V4
     if (strstr(capabilities, "AUTH=KERBEROS_V4"))
@@ -393,12 +670,44 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
     }
 #endif /* KERBEROS_V4 */
 
+    /* this handles either AUTH=LOGIN or AUTH-LOGIN */
+    if ((imap_version >= IMAP4rev1) && (!strstr(capabilities, "LOGIN"))) {
+      error(0,-1, "Required LOGIN capability not supported by server");
+      return PS_AUTHFAIL;
+    };
+
     /* try to get authorized in the ordinary (AUTH=LOGIN) way */
-    ok = gen_transact(sock, "LOGIN %s \"%s\"", ctl->remotename, ctl->password);
-    if (ok)
+    {
+       char *newpass = canonify_imap_password(ctl->password);
+       
+       if (!newpass)
+          return(PS_AUTHFAIL); /* should report error better!!!! */
+       
+       ok = gen_transact(sock, "LOGIN %s \"%s\"", ctl->remotename, newpass);
+       
+       free(newpass);
+    
+       if (ok)
+          return(ok);
+    }
+    
+    return(PS_SUCCESS);
+}
+
+static int internal_expunge(int sock)
+/* ship an expunge, resetting associated counters */
+{
+    int	ok;
+
+    if ((ok = gen_transact(sock, "EXPUNGE")))
 	return(ok);
 
-    return(PS_SUCCESS);
+    expunged += deletions;
+    deletions = 0;
+
+#ifdef IMAP_UID	/* not used */
+    expunge_uids(ctl);
+#endif /* IMAP_UID */
 }
 
 static int imap_getrange(int sock, 
@@ -420,20 +729,25 @@ static int imap_getrange(int sock,
 	 */
 	ok = 0;
 	if (deletions && ctl->expunge > 1)
-	    ok = gen_transact(sock, "EXPUNGE");
-#ifdef IMAP_UID	/* not used */
-	if (!ok)
-	    expunge_uids(ctl);
-#endif /* IMAP_UID */
+	    internal_expunge(sock);
+	count = -1;
 	if (ok || gen_transact(sock, "NOOP"))
 	{
 	    error(0, 0, "re-poll failed");
 	    return(ok);
 	}
+	else if (count == -1)	/* no EXISTS response to NOOP */
+	{
+	    count = recent = 0;
+	    unseen = -1;
+	}
     }
     else
     {
-	ok = gen_transact(sock, "SELECT %s", folder ? folder : "INBOX");
+	if (!check_only)
+	    ok = gen_transact(sock, "SELECT %s", folder ? folder : "INBOX");
+	else
+	    ok = gen_transact(sock, "EXAMINE %s", folder ? folder : "INBOX");
 	if (ok != 0)
 	{
 	    error(0, 0, "mailbox selection failed");
@@ -457,7 +771,7 @@ static int imap_getrange(int sock,
     else
 	*newp = -1;		/* should never happen, RECENT is mandatory */ 
 
-    expunged = deletions = 0;
+    expunged = 0;
 
     return(PS_SUCCESS);
 }
@@ -465,9 +779,17 @@ static int imap_getrange(int sock,
 static int imap_getsizes(int sock, int count, int *sizes)
 /* capture the sizes of all messages */
 {
-    char buf [POPBUFSIZE+1];
+    char buf [MSGBUFSIZE+1];
 
-    gen_send(sock, "FETCH 1:%d RFC822.SIZE", count);
+    /*
+     * Some servers (as in, PMDF5.1-9.1 under OpenVMS 6.1)
+     * won't accept 1:1 as valid set syntax.  Some implementors
+     * should be taken out and shot for excessive anality.
+     */
+    if (count == 1)
+	gen_send(sock, "FETCH 1 RFC822.SIZE", count);
+    else
+	gen_send(sock, "FETCH 1:%d RFC822.SIZE", count);
     for (;;)
     {
 	int num, size, ok;
@@ -500,7 +822,7 @@ static int imap_is_old(int sock, struct query *ctl, int number)
 static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
 /* request headers of nth message */
 {
-    char buf [POPBUFSIZE+1];
+    char buf [MSGBUFSIZE+1];
     int	num;
 
     /* expunges change the fetch numbers */
@@ -530,7 +852,7 @@ static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
 static int imap_fetch_body(int sock, struct query *ctl, int number, int *lenp)
 /* request body of nth message */
 {
-    char buf [POPBUFSIZE+1], *cp;
+    char buf [MSGBUFSIZE+1], *cp;
     int	num;
 
     /* expunges change the fetch numbers */
@@ -596,7 +918,7 @@ static int imap_trail(int sock, struct query *ctl, int number)
 
     for (;;)
     {
-	char buf[POPBUFSIZE+1];
+	char buf[MSGBUFSIZE+1];
 	int ok;
 
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
@@ -628,6 +950,8 @@ static int imap_delete(int sock, struct query *ctl, int number)
 				: "STORE %d +FLAGS (\\Deleted)", 
 			number)))
 	return(ok);
+    else
+	deletions++;
 
     /*
      * We do an expunge after ctl->expunge messages, rather than
@@ -635,17 +959,8 @@ static int imap_delete(int sock, struct query *ctl, int number)
      * won't result in lots of messages being fetched again during
      * the next session.
      */
-    if (ctl->expunge > 0 && (++deletions % ctl->expunge) == 0)
-    {
-	if ((ok = gen_transact(sock, "EXPUNGE")))
-	    return(ok);
-
-#ifdef IMAP_UID	/* not used */
-	expunge_uids(ctl);
-#endif /* IMAP_UID */
-
-	expunged = deletions;
-    }
+    if (NUM_NONZERO(ctl->expunge) && (deletions % ctl->expunge) == 0)
+	internal_expunge(sock);
 
     return(PS_SUCCESS);
 }
@@ -654,17 +969,8 @@ static int imap_logout(int sock, struct query *ctl)
 /* send logout command */
 {
     /* if expunges after deletion have been suppressed, ship one now */
-    if (ctl->expunge == 0 && deletions)
-    {
-	int	ok;
-
-	if ((ok = gen_transact(sock, "EXPUNGE")))
-	    return(ok);
-
-#ifdef IMAP_UID	/* not used */
-	expunge_uids(ctl);
-#endif /* IMAP_UID */
-    }
+    if (NUM_SPECIFIED(ctl->expunge) && NUM_ZERO(ctl->expunge) && deletions)
+	internal_expunge(sock);
 
     return(gen_transact(sock, "LOGOUT"));
 }
@@ -672,7 +978,11 @@ static int imap_logout(int sock, struct query *ctl)
 const static struct method imap =
 {
     "IMAP",		/* Internet Message Access Protocol */
-    143,		/* standard IMAP2bis/IMAP4 port */
+#if INET6
+    "imap",
+#else /* INET6 */
+    143,                /* standard IMAP2bis/IMAP4 port */
+#endif /* INET6 */
     TRUE,		/* this is a tagged protocol */
     FALSE,		/* no message delimiter */
     imap_ok,		/* parse command response */
