@@ -35,25 +35,28 @@
 #define LA_AUTHENTICATE	17
 #define LA_TIMEOUT	18
 #define LA_ENVELOPE	19
-#define LA_USERNAME	20
-#define LA_ALL          21
-#define LA_NOKEEP	22
-#define	LA_KEEP		23
-#define LA_FLUSH        24
-#define LA_NOREWRITE	25
-#define LA_LIMIT	26
-#define LA_FOLDER	27
-#define LA_SMTPHOST	28
-#define LA_BATCHLIMIT	29
-#define LA_FETCHLIMIT	30
-#define LA_EXPUNGE	31
-#define LA_MDA		32
-#define LA_INTERFACE    33
-#define LA_MONITOR      34
-#define LA_YYDEBUG	35
-#define LA_QVIRTUAL     36
+#define LA_QVIRTUAL     20
+#define LA_USERNAME	21
+#define LA_ALL          22
+#define LA_NOKEEP	23
+#define	LA_KEEP		24
+#define LA_FLUSH        25
+#define LA_NOREWRITE	26
+#define LA_LIMIT	27
+#define LA_FOLDER	28
+#define LA_SMTPHOST	29
+#define LA_SMTPADDR     30
+#define LA_ANTISPAM	31
+#define LA_BATCHLIMIT	32
+#define LA_FETCHLIMIT	33
+#define LA_EXPUNGE	34
+#define LA_MDA		35
+#define LA_NETSEC	36
+#define LA_INTERFACE    37
+#define LA_MONITOR      38
+#define LA_YYDEBUG	39
 
-/* options still left: CDgGhHjJoORTUwWxXYzZ */
+/* options still left: CgGhHjJoORUwWxXYzZ */
 static const char *shortoptions = 
 	"?Vcsvd:NqL:f:i:p:UP:A:t:E:Q:u:akKFnl:r:S:b:B:e:m:I:M:y";
 
@@ -86,7 +89,7 @@ static const struct option longoptions[] = {
   {"username",  required_argument, (int *) 0, LA_USERNAME    },
 
   {"all",	no_argument,       (int *) 0, LA_ALL         },
-  {"nokeep",	no_argument,	   (int *) 0, LA_NOKEEP        },
+  {"nokeep",	no_argument,	   (int *) 0, LA_NOKEEP      },
   {"keep",      no_argument,       (int *) 0, LA_KEEP        },
   {"flush",	no_argument,	   (int *) 0, LA_FLUSH       },
   {"norewrite",	no_argument,	   (int *) 0, LA_NOREWRITE   },
@@ -94,15 +97,22 @@ static const struct option longoptions[] = {
 
   {"folder",    required_argument, (int *) 0, LA_FOLDER	     },
   {"smtphost",	required_argument, (int *) 0, LA_SMTPHOST    },
+  {"smtpaddress", required_argument, (int *) 0, LA_SMTPADDR  },
+  {"antispam",	required_argument, (int *) 0, LA_ANTISPAM    },
+  
   {"batchlimit",required_argument, (int *) 0, LA_BATCHLIMIT  },
   {"fetchlimit",required_argument, (int *) 0, LA_FETCHLIMIT  },
   {"expunge",   required_argument, (int *) 0, LA_EXPUNGE     },
   {"mda",	required_argument, (int *) 0, LA_MDA         },
 
-#ifdef	linux
+#ifdef INET6
+  {"netsec",	required_argument, (int *) 0, LA_NETSEC    },
+#endif /* INET6 */
+
+#if defined(linux) && !INET6
   {"interface",	required_argument, (int *) 0, LA_INTERFACE   },
   {"monitor",	required_argument, (int *) 0, LA_MONITOR     },
-#endif
+#endif /* defined(linux) && !INET6 */
 
   {"yydebug",	no_argument,	   (int *) 0, LA_YYDEBUG     },
 
@@ -198,8 +208,16 @@ struct query *ctl;	/* option record to be initialized */
 	    else if (strcasecmp(optarg,"kpop") == 0)
 	    {
 		ctl->server.protocol = P_POP3;
+#if INET6
+		ctl->server.service = KPOP_PORT;
+#else /* INET6 */
 		ctl->server.port = KPOP_PORT;
+#endif /* INET6 */
+#ifdef KERBEROS_V5
+		ctl->server.preauthenticate =  A_KERBEROS_V5;
+#else
 		ctl->server.preauthenticate =  A_KERBEROS_V4;
+#endif /* KERBEROS_V5 */
 	    }
 	    else if (strcasecmp(optarg,"imap") == 0)
 		ctl->server.protocol = P_IMAP;
@@ -220,16 +238,26 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'P':
 	case LA_PORT:
+#if INET6
+	    ctl->server.service = optarg;
+#else /* INET6 */
 	    ctl->server.port = atoi(optarg);
+#endif /* INET6 */
 	    break;
 	case 'A':
 	case LA_AUTHENTICATE:
 	    if (strcmp(optarg, "password") == 0)
 		ctl->server.preauthenticate = A_PASSWORD;
 	    else if (strcmp(optarg, "kerberos") == 0)
+#ifdef KERBEROS_V5
+		ctl->server.preauthenticate = A_KERBEROS_V5;
+	    else if (strcmp(optarg, "kerberos_v5") == 0)
+		ctl->server.preauthenticate = A_KERBEROS_V5;
+#else
 		ctl->server.preauthenticate = A_KERBEROS_V4;
 	    else if (strcmp(optarg, "kerberos_v4") == 0)
 		ctl->server.preauthenticate = A_KERBEROS_V4;
+#endif /* KERBEROS_V5 */
 	    else {
 		fprintf(stderr,"Invalid preauthentication `%s' specified.\n", optarg);
 		errflag++;
@@ -276,9 +304,8 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'l':
 	case LA_LIMIT:
-	    ctl->limit = atoi(optarg);
-	    if (ctl->limit == 0)
-		ctl->limit = -1;
+	    c = atoi(optarg);
+	    ctl->limit = NUM_VALUE(c);
 	    break;
 	case 'r':
 	case LA_FOLDER:
@@ -299,23 +326,28 @@ struct query *ctl;	/* option record to be initialized */
 		((cp = strtok((char *)NULL, ",")));
 	    ocount++;
 	    break;
+	case 'D':
+	case LA_SMTPADDR:
+	    ctl->smtpaddress = xstrdup(optarg);
+	    break;
+	case 'Z':
+	case LA_ANTISPAM:
+	    c = atoi(optarg);
+	    ctl->antispam = NUM_VALUE(c);
 	case 'b':
 	case LA_BATCHLIMIT:
-	    ctl->batchlimit = atoi(optarg);
-	    if (ctl->batchlimit == 0)
-		ctl->batchlimit = -1;
+	    c = atoi(optarg);
+	    ctl->batchlimit = NUM_VALUE(c);
 	    break;
 	case 'B':
 	case LA_FETCHLIMIT:
-	    ctl->fetchlimit = atoi(optarg);
-	    if (ctl->fetchlimit == 0)
-		ctl->fetchlimit = -1;
+	    c = atoi(optarg);
+	    ctl->fetchlimit = NUM_VALUE(c);
 	    break;
 	case 'e':
 	case LA_EXPUNGE:
-	    ctl->expunge = atoi(optarg);
-	    if (ctl->expunge == 0)
-		ctl->expunge = -1;
+	    c = atoi(optarg);
+	    ctl->expunge = NUM_VALUE(c);
 	    break;
 	case 'm':
 	case LA_MDA:
@@ -323,7 +355,17 @@ struct query *ctl;	/* option record to be initialized */
 	    ocount++;
 	    break;
 
-#ifdef	linux
+	case 'T':
+	case LA_NETSEC:
+#if NET_SECURITY
+	    ctl->server.netsec = (void *)optarg;
+#else
+	    fprintf(stderr, "fetchmail: network security support is disabled\n");
+	    errflag++;
+#endif /* NET_SECURITY */
+	    break;
+
+#if defined(linux) && !INET6
 	case 'I':
 	case LA_INTERFACE:
 	    interface_parse(optarg, &ctl->server);
@@ -332,7 +374,7 @@ struct query *ctl;	/* option record to be initialized */
 	case LA_MONITOR:
 	    ctl->server.monitor = xstrdup(optarg);
 	    break;
-#endif
+#endif /* defined(linux) && !INET6 */
 
 	case 'y':
 	case LA_YYDEBUG:
@@ -368,7 +410,7 @@ struct query *ctl;	/* option record to be initialized */
 	fputs("      --invisible   suppress Received line & enable host spoofing\n", stderr);
 	fputs("  -f, --fetchmailrc specify alternate run control file\n", stderr);
 	fputs("  -i, --idfile      specify alternate UIDs file\n", stderr);
-#ifdef	linux
+#if defined(linux) && !INET6
 	fputs("  -I, --interface   interface required specification\n",stderr);
 	fputs("  -M, --monitor     monitor interface for activity\n",stderr);
 #endif
@@ -393,7 +435,12 @@ struct query *ctl;	/* option record to be initialized */
 	fputs("  -n, --norewrite   don't rewrite header addresses\n", stderr);
 	fputs("  -l, --limit       don't fetch messages over given size\n", stderr);
 
+#if NET_SECURITY
+	fputs("  -T, --netsec      set IP security request\n", stderr);
+#endif /* NET_SECURITY */
 	fputs("  -S, --smtphost    set SMTP forwarding host\n", stderr);
+	fputs("  -D, --smtpaddress set SMTP delivery domain to use\n", stderr);
+	fputs("  -Z, --antispam,   set antispam response value\n", stderr);
 	fputs("  -b, --batchlimit  set batch limit for SMTP connections\n", stderr);
 	fputs("  -B, --fetchlimit  set fetch limit for server connections\n", stderr);
 	fputs("  -e, --expunge     set max deletions between expunges\n", stderr);
