@@ -28,6 +28,49 @@
 #endif
 #include "socket.h"
 
+#if NET_SECURITY
+#include <net/security.h>
+#endif /* NET_SECURITY */
+
+#if INET6
+int SockOpen(const char *host, const char *service, const char *options)
+{
+  int i;
+  struct addrinfo *ai, req;
+#if NET_SECURITY
+  void *request = NULL;
+  int requestlen;
+#endif /* NET_SECURITY */
+
+  memset(&req, 0, sizeof(struct addrinfo));
+  req.ai_socktype = SOCK_STREAM;
+
+  if (i = getaddrinfo(host, service, &req, &ai)) {
+    fprintf(stderr, "fetchmail: getaddrinfo(%s.%s): %s(%d)\n", host, service, gai_strerror(i), i);
+    return -1;
+  };
+
+#if NET_SECURITY
+  if (!options)
+    requestlen = 0;
+  else
+    if (net_security_strtorequest((char *)options, &request, &requestlen))
+      goto ret;
+
+  i = inner_connect(ai, request, requestlen, NULL,NULL, "fetchmail", NULL);
+  if (request)
+    free(request);
+
+ret:
+#else /* NET_SECURITY */
+  i = inner_connect(ai, NULL, 0, NULL, NULL, "fetchmail", NULL);
+#endif /* NET_SECURITY */
+
+  freeaddrinfo(ai);
+
+  return i;
+};
+#else /* INET6 */
 #ifndef INET_ATON
 #ifndef  INADDR_NONE
 #ifdef   INADDR_BROADCAST
@@ -38,7 +81,7 @@
 #endif
 #endif /* INET_ATON */
 
-int SockOpen(const char *host, int clientPort)
+int SockOpen(const char *host, int clientPort, const char *options)
 {
     int sock;
 #ifndef INET_ATON
@@ -69,6 +112,12 @@ int SockOpen(const char *host, int clientPort)
         if (hp == NULL || (hp->h_length != 4 && hp->h_length != 8))
             return -1;
 
+	/*
+	 * FIXME: make this work for multihomed hosts.
+	 * We're toast if we get back multiple addresses and h_addrs[0]
+	 * (aka h_addr) is not one we can actually connect to; this happens
+	 * with multi-homed boxen.
+	 */
         memcpy(&ad.sin_addr, hp->h_addr, hp->h_length);
     }
     ad.sin_port = htons(clientPort);
@@ -84,6 +133,7 @@ int SockOpen(const char *host, int clientPort)
 
     return(sock);
 }
+#endif /* INET6 */
 
 
 #if defined(HAVE_STDARG_H)
@@ -178,7 +228,7 @@ int SockPeek(int sock)
  */
 main()
 {
-    int	 	sock = SockOpen("localhost", 19);
+    int	 	sock = SockOpen("localhost", 19, NULL);
     char	buf[80];
 
     while (SockRead(sock, buf, sizeof(buf)-1))
